"""
    domonic.webapi.push
    ====================================
    https://developer.mozilla.org/en-US/docs/Web/API/Push_API
"""


# class PushManager:
#     """
#     The PushManager interface provides a way for a web application to
#     register for push notifications.
#     """
#     def getSubscription(self):
#         """
#         Returns the subscription object for the current document.
#         """
#         raise NotImplementedError()

#     def permissionState(self):
#         """
#         Returns the permission state of the current document.
#         """
#         raise NotImplementedError()

#     def subscribe(self, options):
#         """
#         Subscribes the document to push notifications.
#         """
#         raise NotImplementedError()

#     def unsubscribe(self):
#         """
#         Unsubscribes the document from push notifications.
#         """
#         raise NotImplementedError()